#! /usr/bin/php -f

<?php
function mmcache_encoder_usage() {
  echo "Usage:\tphp -q encoder.php [options] source_file_name\n";
  echo       "\tphp -q encoder.php [options] source_file_name...\n";
  echo       "\tphp -q encoder.php [options] source_directory_name...\n\n";
  echo "Options:\n";
  echo "\t-s suffix\n\t\tencode files only with following suffix (default is \"php\")\n";
  echo "\t-a\n\t\tencode all files (no by default)\n";
  echo "\t-l\n\t\tfollow symbolic links (no by default)\n";
  echo "\t-r\n\t\tencode directories recursively (no by default)\n";
  echo "\t-c\n\t\tcopy files those shouldn't be encoded (no by default)\n";
  echo "\t-f\n\t\toverwrite existing files (no by default)\n";
  echo "\t-o target\n\t\tIf you encode only one script then 'target' specifyes an output\n";
  echo               "\t\tfile name. If you encode directory or several files at once\n";
  echo               "\t\tthen 'target' specifyes an output directory name.\n";
  echo "\nExamples:\n";
  echo "\tphp -q encoder.php some_file.php\n";
  echo "\tphp -q encoder.php some_file.php -o some_encoded_file.php\n";
  echo "\tphp -q encoder.php *.php -o some_dir\n";
  echo "\tphp -q encoder.php ~/public_html/x -rcf -sphp -sinc -o ~/public_html/y\n";
  echo "\n";
  exit();
}

function mmcache_encode_file($src, $out, $f, $c) {
  if (empty($out)) {
    echo "\n// $src\n";
  }
  if ($f = @fopen($src,"r")) {
    $prefix = fgets($f,4096);
    if ($prefix[0] != '#') {
      $prefix = "";
      rewind($f);
    }
    fclose($f);
  }
  $cmp = mmcache_encode($src);
  if (empty($cmp)) {
    echo("MMCache Encoder ERROR: Can't compile file \"$src\"\n");
    if ($f) {
      if ($c && !empty($out)) {
        mmcache_copy_file($src, $out, $f);
      }
    } else {
      exit;
    }
  } else {
    $cmp = $prefix."<?php return mmcache_load('".$cmp."');?>\n";
    if (!empty($out)) {
      if (!$f && file_exists($out)) {
        echo("MMCache Encoder ERROR: Can't create output file \"$out\" (already exists).\n");
      } else {
        $file = @fopen($out,"wb");
        if (!$file) {
          echo("MMCache Encoder ERROR: Can't open output file \"$out\"\n");
        } else {
          fwrite($file,$cmp);
          unset($cmp);
          fclose($file);
          $stat = stat($src);
          chmod($out, $stat['mode']);
        }
      }
    } else {
      echo $cmp;
      unset($cmp);
    }
  }
}

function mmcache_copy_dir($src, $dir, $f) {
  $stat = stat($src);
  $old = umask(0);
  $ret = mmcache_mkdir($dir, $f);
  umask($old);
  return $ret;
}

function mmcache_mkdir($dir, $f) {
  if (!empty($dir)) {
    if (!@mkdir($dir,0777)) {
      if (!$f) {
        echo("MMCache Encoder ERROR: Can't create destination directory \"$dir\"");
        if (file_exists($dir)) {
          echo " (already exists)";
        }
        echo ".\n";
        return 0;
      }
    }
  }
  return 1;
}

function mmcache_copy_file($src, $out, $f) {
  $i = @fopen($src, "rb");
  if (!$i) {
    echo("MMCache Encoder ERROR: Can't open file \"$src\" for copying.\n");
    return;
  }
  if (!$f && file_exists($out)) {
    echo("MMCache Encoder ERROR: Can't create output file \"$out\" (already exists).\n");
  } else {
    $o = @fopen($out, "wb");
    if (!$o) {
      echo("MMCache Encoder ERROR: Can't copy file into \"$out\".\n");
      return;
    }
    while ($tmp = fread($i, 1024*32)) {
      fwrite($o, $tmp);
    }
    fclose($i);
    fclose($o);
    $stat = stat($src);
    chmod($out, $stat['mode']);
  }
}

function mmcache_copy_link($src, $out, $f) {
  $link = readlink($src);
  if (!@symlink($link,$out)) {
    if ($f && file_exists($out)) {
      unlink($out);
      mmcache_copy_link($src, $out, false);
    } else if ($f && is_array(lstat($out))) {
      unlink($out);
      mmcache_copy_link($src, $out, false);
    } else {
      echo("MMCache Encoder ERROR: Can't create symlink \"$out\" -> \"$link\".\n");
      return;
    }
  }
}

function mmcache_encode_dir($src, $out, $s, $r, $l, $c, $f) {
  if ($dir = @opendir($src)) {
    while (($file = readdir($dir)) !== false) {
      if ($file == "." || $file == "..") continue;
      $i = "$src/$file";
      $o = empty($out)?$out:"$out/$file";
      if (is_link($i)) {
        if ($c && !empty($o)) {
          mmcache_copy_link($i, $o, $f);
          continue;
        } else if (!$l) {
          continue;
        }
      }
      if (is_dir($i)) {
        if ($r) {
          if (mmcache_copy_dir($i, $o, $f)) {
            mmcache_encode_dir($i, $o, $s, $r, $l, $c, $f);
          }
        }
      } else if (is_file($i)) {
        if (empty($s)) {
          mmcache_encode_file($i, $o, $f, $c);
        } else if (is_string($s)) {
          if (preg_match("/".preg_quote(".$s")."\$/i", $file)) {
            mmcache_encode_file($i, $o, $f, $c);
          } else if (!empty($o)) {
            mmcache_copy_file($i, $o, $f, $c);
          }
        } else if (is_array($s)) {
          $encoded = false;
          foreach($s as $z) {
            if (preg_match("/".preg_quote(".$z")."\$/i", $file)) {
              mmcache_encode_file($i, $o, $f, $c);
              $encoded = true;
              break;
            }
          }
          if (!$encoded && !empty($o)) {
            mmcache_copy_file($i, $o, $f);
          }
        }
      }
    }
    closedir($dir);
  } else {
    echo("MMCache Encoder ERROR: Can't open source directory \"$src\".\n");
  }
}

function mmcache_encoder_main() {
  $argc = $_SERVER['argc'];
  $argv = $_SERVER['argv'];

  $src = array();
  $out = null;
  unset($s);
  $r = false;
  $l = false;
  $a = false;
  $c = false;
  $f = false;

  for ($i = 1; $i < $argc; $i++) {
    $arg = $argv[$i];
    if (!empty($arg)) {
      if ($arg[0] == '-') {
        if ($arg[1] == "o") {
          if (!empty($out)) {
            mmcache_encoder_usage();
          }
          if (strlen($arg) == 2) {
            if ($argc <= $i || empty($argv[$i+1]) || $argv[$i+1][0] == "-") {
              mmcache_encoder_usage();
            }
            $out = $argv[++$i];
          } else {
            $out = substr($arg,2);
          }

        } else if ($arg[1] == "s") {
          if (strlen($arg) == 2) {
            $s[] = $argv[++$i];
          } else {
            $s[] = substr($arg,2);
          }
        } else if (strlen($arg) == 2 && $arg[1] == "r") {
          $r = true;
        } else if (strlen($arg) == 2 && $arg[1] == "l") {
          $l = true;
        } else if (strlen($arg) == 2 && $arg[1] == "a") {
          $a = true;
        } else if (strlen($arg) == 2 && $arg[1] == "c") {
          $c = true;
        } else if (strlen($arg) == 2 && $arg[1] == "f") {
          $f = true;
        } else {
          $len = strlen($arg);
          if ($len > 1) {
            $n   = 1;
            while ($n < $len) {
              if ($arg[$n] == "r") {
                $r = true;
              } else if ($arg[$n] == "l") {
                $l = true;
              } else if ($arg[$n] == "a") {
                $a = true;
              } else if ($arg[$n] == "c") {
                $c = true;
              } else if ($arg[$n] == "f") {
                $f = true;
              } else {
                if ($arg[$n] != "o" && $arg[$n] != "s") {
                  echo("MMCache Encoder ERROR: Unknown option \"-".$arg[$n]."\"\n\n");
                }
                mmcache_encoder_usage();
              }
              ++$n;
            }
          } else {
            echo("MMCache Encoder ERROR: Unknown option \"$arg\"\n\n");
            mmcache_encoder_usage();
          }
        }
      } else {
        $src[] = $arg;
      }
    }
  }
  if (isset($src) && is_array($src) && count($src) > 0) {
    $cnt = count($src);
    if ($a) {
      unset($s);
    } else if (!isset($s)) {
      $s = "php";
    }
    if ($cnt > 1) {
      if (!mmcache_mkdir($out, $f)) {
        return;
      }
    }
    foreach($src as $file) {
      if (!file_exists($file)) {
        echo("MMCache Encoder ERROR: Source file \"$file\" doesn't exist.\n");
      } else {
        if (is_dir($file)) {
          if ($cnt == 1) {
            if (mmcache_mkdir($out, $f)) {
              mmcache_encode_dir($file, $out, $s, $r, $l, $c, $f);
            }
          } else {
            if (mmcache_copy_dir($file, $out."/".basename($file), $f)) {
              mmcache_encode_dir($file, $out."/".basename($file), $s, $r, $l, $c, $f);
            }
          }
        } else {
          if ($cnt == 1) {
            mmcache_encode_file($file, $out, $f, $c);
          } else {
            if (empty($out)) {
              mmcache_encode_file($file, $out, $f, $c);
            } else {
              mmcache_encode_file($file, $out."/".basename($file), $f, $c);
            }
          }
        }
      }
    }
  } else {
    mmcache_encoder_usage();
  }
}

//function mmcache_encoder_web() {
//  echo "<html>\n<head>\n<title>Turck MMCache Encoder</title>\n</head>\n<body>\n";
//  echo "<h1 align=\"center\">Turck MMCache Encoder</h1>\n";
//  echo "Turck MMCache Encoder WEB interface is not implemented use \"<b>encoder.php</b>\" script from command line.<br><br><b>php -q encoder.php</b>\n";
//  echo "</body>\n</html>\n";
//}

set_time_limit(0);
//if (!empty($_SERVER['PHP_SELF'])) {
//  if (function_exists("mmcache_encode")) {
//    mmcache_encoder_web();
//  } else {
//    echo "Turck MMCache Encoder is not installed\n";
//  }
//} else {
  if (function_exists("mmcache_encode")) {
    mmcache_encoder_main();
  } else {
    echo "<html><head><title>Turck MMCache Encoder</title></head><body><h1 align=\"center\">Turck MMCache Encoder is not installed</h1></body></html>";
  }
//}
?>
